@@ -92,6 +92,8 @@ struct _PeasEnginePrivate {
 
   GList *plugin_list;
   GHashTable *loaders;
+
+  guint in_dispose : 1;
 };
 
 static void peas_engine_load_plugin_real   (PeasEngine     *engine,
@@ -170,8 +172,8 @@ load_dir_real (PeasEngine  *engine,
  *
  * Rescan all the registered directories to find new or updated plugins.
  *
- * Calling this function will make the newly installed plugin infos to be
- * loaded by the engine, so the new plugins can actually be used without
+ * Calling this function will make the newly installed plugin infos
+ * be loaded by the engine, so the new plugins can be used without
  * restarting the application.
  */
 void
@@ -206,11 +208,11 @@ peas_engine_rescan_plugins (PeasEngine *engine)
  * This function appends a search path to the list of paths where to
  * look for plugins.
  *
- * A so-called "search path" actually consists of a couple of both a
+ * A so-called "search path" actually consists of both a
  * module directory (where the shared libraries or language modules
  * lie) and a data directory (where the plugin data is).
  *
- * The #PeasPlugin will be able to use a correct data dir depending on
+ * The plugin will be able to use a correct data dir depending on
  * where it is installed, hence allowing to keep the plugin agnostic
  * when it comes to installation location: the same plugin can be
  * installed either in the system path or in the user's home directory,
@@ -309,6 +311,8 @@ peas_engine_init (PeasEngine *engine)
                                               PEAS_TYPE_ENGINE,
                                               PeasEnginePrivate);
 
+  engine->priv->in_dispose = FALSE;
+
   /* mapping from loadername -> loader object */
   engine->priv->loaders = g_hash_table_new_full (hash_lowercase,
                                                  (GEqualFunc) equal_lowercase,
@@ -316,12 +320,6 @@ peas_engine_init (PeasEngine *engine)
                                                  (GDestroyNotify) loader_destroy);
 }
 
-static void
-peas_engine_constructed (GObject *object)
-{
-  peas_engine_rescan_plugins (PEAS_ENGINE (object));
-}
-
 static void
 loader_garbage_collect (const gchar *id,
                         LoaderInfo  *info)
@@ -392,22 +390,28 @@ peas_engine_get_property (GObject    *object,
 }
 
 static void
-peas_engine_finalize (GObject *object)
+peas_engine_dispose (GObject *object)
 {
   PeasEngine *engine = PEAS_ENGINE (object);
   GList *item;
 
+  engine->priv->in_dispose = TRUE;
+
   /* First unload all the plugins */
   for (item = engine->priv->plugin_list; item; item = item->next)
     {
       PeasPluginInfo *info = PEAS_PLUGIN_INFO (item->data);
 
       if (peas_plugin_info_is_loaded (info))
-        peas_engine_unload_plugin_real (engine, info);
+        peas_engine_unload_plugin (engine, info);
     }
 
   /* unref the loaders */
-  g_hash_table_destroy (engine->priv->loaders);
+  if (engine->priv->loaders != NULL)
+    {
+      g_hash_table_destroy (engine->priv->loaders);
+      engine->priv->loaders = NULL;
+    }
 
   /* and finally free the infos */
   for (item = engine->priv->plugin_list; item; item = item->next)
@@ -423,6 +427,14 @@ peas_engine_finalize (GObject *object)
       g_slice_free (SearchPath, sp);
     }
 
+  G_OBJECT_CLASS (peas_engine_parent_class)->dispose (object);
+}
+
+static void
+peas_engine_finalize (GObject *object)
+{
+  PeasEngine *engine = PEAS_ENGINE (object);
+
   g_list_free (engine->priv->plugin_list);
   g_list_free (engine->priv->search_paths);
 
@@ -437,7 +449,7 @@ peas_engine_class_init (PeasEngineClass *klass)
 
   object_class->set_property = peas_engine_set_property;
   object_class->get_property = peas_engine_get_property;
-  object_class->constructed = peas_engine_constructed;
+  object_class->dispose = peas_engine_dispose;
   object_class->finalize = peas_engine_finalize;
 
   klass->load_plugin = peas_engine_load_plugin_real;
@@ -478,6 +490,8 @@ peas_engine_class_init (PeasEngineClass *klass)
    *                    "loaded-plugins",
    *                    G_SETTINGS_BIND_DEFAULT);
    * ]|
+   *
+   * Note: notify will not be called when the engine is being destroyed.
    */
   g_object_class_install_property (object_class,
                                    PROP_LOADED_PLUGINS,
@@ -708,7 +722,8 @@ compare_plugin_info_and_name (PeasPluginInfo *info,
  * @engine: A #PeasEngine.
  * @plugin_name: A plugin name.
  *
- * Returns: the #PeasPluginInfo corresponding with a given plugin name.
+ * Returns: the #PeasPluginInfo corresponding with
+ * a given plugin module name.
  */
 PeasPluginInfo *
 peas_engine_get_plugin_info (PeasEngine  *engine,
@@ -747,11 +762,11 @@ load_plugin (PeasEngine     *engine,
       if (!dep_info)
         {
           g_warning ("Plugin not found: %s", dependencies[i]);
-          return FALSE;
+          goto error;
         }
 
       if (!peas_engine_load_plugin (engine, dep_info))
-        return FALSE;
+        goto error;
     }
 
   loader = get_plugin_loader (engine, info);
@@ -760,19 +775,23 @@ load_plugin (PeasEngine     *engine,
     {
       g_warning ("Could not find loader '%s' for plugin '%s'",
                  info->loader, info->name);
-      info->available = FALSE;
-      return FALSE;
+      goto error;
     }
 
   if (!peas_plugin_loader_load (loader, info))
     {
       g_warning ("Error loading plugin '%s'", info->name);
-      info->loaded = FALSE;
-      info->available = FALSE;
-      return FALSE;
+      goto error;
     }
 
   return TRUE;
+
+error:
+
+  info->loaded = FALSE;
+  info->available = FALSE;
+
+  return FALSE;
 }
 
 static void
@@ -846,7 +865,8 @@ peas_engine_unload_plugin_real (PeasEngine     *engine,
   peas_plugin_loader_garbage_collect (loader);
   peas_plugin_loader_unload (loader, info);
 
-  g_object_notify (G_OBJECT (engine), "loaded-plugins");
+  if (!engine->priv->in_dispose)
+    g_object_notify (G_OBJECT (engine), "loaded-plugins");
 }
 
 /**
@@ -897,7 +917,8 @@ peas_engine_provides_extension (PeasEngine     *engine,
  * @info: A loaded #PeasPluginInfo.
  * @extension_type: The implemented extension #GType.
  * @n_parameters: the length of the @parameters array.
- * @parameters: an array of #GParameter.
+ * @parameters: (allow-none) (array length=n_parameters):
+ *   an array of #GParameter.
  *
  * If the plugin identified by @info implements the @extension_type interface,
  * then this function will return a new instance of this implementation,
@@ -907,6 +928,8 @@ peas_engine_provides_extension (PeasEngine     *engine,
  *
  * Returns: (transfer full): a new instance of #PeasExtension wrapping
  * the @extension_type instance, or %NULL.
+ *
+ * Rename to: peas_engine_create_extension
  */
 PeasExtension *
 peas_engine_create_extensionv (PeasEngine     *engine,
@@ -926,7 +949,7 @@ peas_engine_create_extensionv (PeasEngine     *engine,
 }
 
 /**
- * peas_engine_create_extension_valist:
+ * peas_engine_create_extension_valist: (skip)
  * @engine: A #PeasEngine.
  * @info: A loaded #PeasPluginInfo.
  * @extension_type: The implemented extension #GType.
@@ -940,7 +963,7 @@ peas_engine_create_extensionv (PeasEngine     *engine,
  *
  * See peas_engine_create_extension() for more information.
  *
- * Returns: (transfer full): a new instance of #PeasExtension wrapping
+ * Returns: a new instance of #PeasExtension wrapping
  * the @extension_type instance, or %NULL.
  */
 PeasExtension *
@@ -976,7 +999,7 @@ peas_engine_create_extension_valist (PeasEngine     *engine,
 }
 
 /**
- * peas_engine_create_extension:
+ * peas_engine_create_extension: (skip)
  * @engine: A #PeasEngine.
  * @info: A loaded #PeasPluginInfo.
  * @extension_type: The implemented extension #GType.
@@ -997,7 +1020,7 @@ peas_engine_create_extension_valist (PeasEngine     *engine,
  * principle of never giving you the actual object (also because it might as
  * well *not* be an actual object).
  *
- * Returns: (transfer full): a new instance of #PeasExtension wrapping
+ * Returns: a new instance of #PeasExtension wrapping
  * the @extension_type instance, or %NULL.
  */
 PeasExtension *
@@ -1049,7 +1072,16 @@ peas_engine_get_loaded_plugins (PeasEngine *engine)
         }
     }
 
-  return (gchar **) g_array_free (array, FALSE);
+  /* Bug in GArray? */
+  if (array->len == 0)
+    {
+      g_array_free (array, TRUE);
+      return NULL;
+    }
+  else
+    {
+      return (gchar **) g_array_free (array, FALSE);
+    }
 }
 
 static gboolean
@@ -1057,6 +1089,10 @@ string_in_strv (const gchar  *needle,
                 const gchar **haystack)
 {
   guint i;
+
+  if (haystack == NULL)
+    return FALSE;
+
   for (i = 0; haystack[i] != NULL; i++)
     if (strcmp (haystack[i], needle) == 0)
       return TRUE;
