@@ -29,6 +29,7 @@
 #include "peas-plugin-info.h"
 #include "peas-marshal.h"
 #include "peas-helpers.h"
+#include "peas-introspection.h"
 
 /**
  * SECTION:peas-extension-set
@@ -115,6 +116,11 @@ typedef struct {
   PeasExtension *exten;
 } ExtensionItem;
 
+typedef struct {
+  guint n_parameters;
+  GParameter *parameters;
+} PeasParameterArray;
+
 /* Signals */
 enum {
   EXTENSION_ADDED,
@@ -282,13 +288,22 @@ peas_extension_set_constructed (GObject *object)
 }
 
 static void
-peas_extension_set_finalize (GObject *object)
+peas_extension_set_dispose (GObject *object)
 {
   PeasExtensionSet *set = PEAS_EXTENSION_SET (object);
   GList *l;
 
-  g_signal_handler_disconnect (set->priv->engine, set->priv->load_handler_id);
-  g_signal_handler_disconnect (set->priv->engine, set->priv->unload_handler_id);
+  if (set->priv->load_handler_id != 0)
+    {
+      g_signal_handler_disconnect (set->priv->engine, set->priv->load_handler_id);
+      set->priv->load_handler_id = 0;
+    }
+
+  if (set->priv->unload_handler_id != 0)
+    {
+      g_signal_handler_disconnect (set->priv->engine, set->priv->unload_handler_id);
+      set->priv->unload_handler_id = 0;
+    }
 
   for (l = set->priv->extensions; l;)
     {
@@ -300,25 +315,31 @@ peas_extension_set_finalize (GObject *object)
     {
       while (set->priv->n_parameters-- > 0)
         g_value_unset (&set->priv->parameters[set->priv->n_parameters].value);
+
       g_free (set->priv->parameters);
+      set->priv->parameters = NULL;
     }
 
-  g_object_unref (set->priv->engine);
+  if (set->priv->engine != NULL)
+    {
+      g_object_unref (set->priv->engine);
+      set->priv->engine = NULL;
+    }
 }
 
 static gboolean
 peas_extension_set_call_real (PeasExtensionSet *set,
-                              const gchar      *method,
-                              va_list           args)
+                              const gchar      *method_name,
+                              GIArgument       *args)
 {
   gboolean ret = TRUE;
   GList *l;
-  va_list args_copy;
+  GIArgument dummy;
 
   for (l = set->priv->extensions; l; l = l->next)
     {
-      G_VA_COPY (args_copy, args);
-      ret = peas_extension_call_valist (((ExtensionItem *) l->data)->exten, method, args_copy) && ret;
+      ExtensionItem *item = (ExtensionItem *) l->data;
+      ret = peas_extension_callv (item->exten, method_name, args, &dummy) && ret;
     }
 
   return ret;
@@ -333,7 +354,7 @@ peas_extension_set_class_init (PeasExtensionSetClass *klass)
   object_class->set_property = peas_extension_set_set_property;
   object_class->get_property = peas_extension_set_get_property;
   object_class->constructed = peas_extension_set_constructed;
-  object_class->finalize = peas_extension_set_finalize;
+  object_class->dispose = peas_extension_set_dispose;
 
   klass->call = peas_extension_set_call_real;
 
@@ -372,7 +393,8 @@ peas_extension_set_class_init (PeasExtensionSetClass *klass)
    *
    * The extension-added signal is emitted when a new extension is about to be
    * removed from the #PeasExtensionSet. It happens when a plugin implementing
-   * the extension set's extension type is unloaded.
+   * the extension set's extension type is unloaded, or when the
+   * #PeasExtensionSet itself is destroyed.
    *
    * You should connect to this signal in order to clean up the extensions
    * when their plugin is unload. Note that this signal is not fired for the
@@ -421,7 +443,37 @@ peas_extension_set_class_init (PeasExtensionSetClass *klass)
 }
 
 /**
- * peas_extension_set_call:
+ * peas_extension_set_get_extension:
+ * @set: A #PeasExtensionSet
+ * @info: a #PeasPluginInfo
+ *
+ * Returns the #PeasExtension object corresponding to @info, or %NULL
+ * if the plugin doesn't provide such an extension.
+ *
+ * Returns: (transfer none): a reference to a #PeasExtension or %NULL
+ */
+PeasExtension *
+peas_extension_set_get_extension (PeasExtensionSet *set,
+                                  PeasPluginInfo   *info)
+{
+  GList *l;
+
+  g_return_val_if_fail (PEAS_IS_EXTENSION_SET (set), NULL);
+  g_return_val_if_fail (info != NULL, NULL);
+
+  for (l = set->priv->extensions; l != NULL; l = l->next)
+    {
+      ExtensionItem *item = l->data;
+
+      if (item->info == info)
+        return item->exten;
+    }
+
+  return NULL;
+}
+
+/**
+ * peas_extension_set_call: (skip)
  * @set: A #PeasExtensionSet.
  * @method_name: the name of the method that should be called.
  * @Varargs: arguments for the method.
@@ -448,10 +500,10 @@ peas_extension_set_call (PeasExtensionSet *set,
 }
 
 /**
- * peas_extension_set_call_valist:
+ * peas_extension_set_call_valist: (skip)
  * @set: A #PeasExtensionSet.
  * @method_name: the name of the method that should be called.
- * @args: the arguments for the method.
+ * @va_args: the arguments for the method.
  *
  * Call a method on all the #PeasExtension instances contained in @set.
  *
@@ -462,7 +514,47 @@ peas_extension_set_call (PeasExtensionSet *set,
 gboolean
 peas_extension_set_call_valist (PeasExtensionSet *set,
                                 const gchar      *method_name,
-                                va_list           args)
+                                va_list           va_args)
+{
+  GICallableInfo *callable_info;
+  GIArgument *args;
+  gint n_args;
+
+  g_return_val_if_fail (PEAS_IS_EXTENSION_SET (set), FALSE);
+  g_return_val_if_fail (method_name != NULL, FALSE);
+
+  callable_info = peas_gi_get_method_info (set->priv->exten_type, method_name);
+  g_return_val_if_fail (callable_info != NULL, FALSE);
+
+  n_args = g_callable_info_get_n_args (callable_info);
+  g_return_val_if_fail (n_args >= 0, FALSE);
+
+  args = g_newa (GIArgument, n_args);
+  peas_gi_valist_to_arguments (callable_info, va_args, args, NULL);
+
+  g_base_info_unref ((GIBaseInfo *) callable_info);
+
+  return peas_extension_set_callv (set, method_name, args);
+}
+
+/**
+ * peas_extension_set_callv:
+ * @set: A #PeasExtensionSet.
+ * @method_name: the name of the method that should be called.
+ * @args: the arguments for the method.
+ *
+ * Call a method on all the #PeasExtension instances contained in @set.
+ *
+ * See peas_extension_callv() for more information.
+ *
+ * Return value: %TRUE on successful call.
+ *
+ * Rename to: peas_extension_set_call
+ */
+gboolean
+peas_extension_set_callv (PeasExtensionSet *set,
+                          const gchar      *method_name,
+                          GIArgument       *args)
 {
   PeasExtensionSetClass *klass;
 
@@ -478,13 +570,15 @@ peas_extension_set_call_valist (PeasExtensionSet *set,
  * @engine: A #PeasEngine.
  * @exten_type: the extension #GType.
  * @n_parameters: the length of the @parameters array.
- * @parameters: an array of #GParameter.
+ * @parameters: (array length=n_parameters): an array of #GParameter.
  *
  * Create a new #PeasExtensionSet for the @exten_type extension type.
  *
  * See peas_extension_set_new() for more information.
  *
- * Returns: a new instance of #PeasExtensionSet.
+ * Returns: (transfer full): a new instance of #PeasExtensionSet.
+ *
+ * Rename to: peas_extension_set_new
  */
 PeasExtensionSet *
 peas_extension_set_newv (PeasEngine *engine,
@@ -502,7 +596,7 @@ peas_extension_set_newv (PeasEngine *engine,
 }
 
 /**
- * peas_extension_set_new_valist:
+ * peas_extension_set_new_valist: (skip)
  * @engine: A #PeasEngine.
  * @exten_type: the extension #GType.
  * @first_property: the name of the first property.
@@ -547,7 +641,7 @@ peas_extension_set_new_valist (PeasEngine  *engine,
 }
 
 /**
- * peas_extension_set_new:
+ * peas_extension_set_new: (skip)
  * @engine: A #PeasEngine.
  * @exten_type: the extension #GType.
  * @first_property: the name of the first property.
